local config_name = "MDI RoidDB"
declare("roiddb", {
	config_name = config_name,
	version = "1.0.0",
	notesid = 7643320, -- pinned roids
	--GetAsteroidData,
})

local T = {
	Events = {},
	FONT_SCALE = 1,
	Font = Font.H5 * 1, --(HUD_SCALE or 1),
	disable_auto_close = gkini.ReadInt(config_name, "Disable Auto Close", 0) == 1,
	load_tab = gkini.ReadInt(config_name, "Load Tab", 1) == 1,
	HighestSystemID = 33,
	LocalData = {
		pinned = {},
	},
	Scan = {
		AutoScanStartStop,
		GetScannerRange,
		GetSectorNodeID,
		Lock,
		LockFunction,
		ScannedRoids,
		TargetNextClosestRoid,
		UnscannedRoids,
		-- -----------------
		lock = false,
		ningAuto, -- Use if auto-scan is in progress.  Referenced in event TARGET_SCANNED to get the next target
		re = gkini.ReadInt(config_name, "Rescan Asteroids", 0) == 1, -- rescan asteroids
		range = -1,
		roid_list = {},
		timer = Timer(),
	},
	-- RoidData
	Save = {
		timer = Timer{},
		interval = gkini.ReadInt(config_name, "Save Interval", 60) * 1000,
	},
	Tab = {}, -- MDI-TargetList tab integration.  Defined in targetlist.lua
	UI = {},
	MessageDlg = msgdlgtemplate1(),
}
roiddb.T = T

console_print("MDI-RoidDB loaded (v"..roiddb.version..").")

--  TODO LIST
-- - Export data Prospector/TargetLess

-- - Next Version???
-- - remember sort order on re-open
-- - same bound button to open/close
-- - customizable tab scanner settings
-- - multi-point nav routes
-- - Added Stats page
-- - Mark a roid sector as hostile
-- - clean up searchmin functionality

function T.IsMobile(tf) return tf or Platform == "Android" or Platform == "iOS" end

function T:SaveSettings()
	gkini.WriteInt(config_name, "Save Interval", T.Save.interval / 1000)
	gkini.WriteInt(config_name, "Rescan Asteroids", T.Scan.re and 1 or 0)
	gkini.WriteInt(config_name, "Disable Auto Close", T.disable_auto_close and 1 or 0)
	gkini.WriteInt(config_name, "Load Tab", T.load_tab and 1 or 0)
end

local function check_number(num, Lval, Uval, dp)
	-- verify a number is within the given range
	-- <num> the number to check
	-- <Lval>, <Uval>: Lower and Upper bounds of the number range; if missing, compares the number against itself
	-- [dp]:  how many decimal places to consider
	num = tonumber(num)
	Lval = tonumber(Lval) or num
	Uval = tonumber(Uval) or num
	if (num and Lval <= num and Uval >= num) then
		if (dp) then
			local decimals = tostring(num):match("%.(%d+)") or "0"
			if (decimals and decimals:len() <= dp) then
				return num -- entered number has no more decimal places than specified
			else
				return false -- too many decimal places
			end
		else
			return num -- non-decimal number is within range
		end
	end
	return false  -- you done fucked up
end

local function loadluafile(...)
    local f, err = loadfile((...))
    assert(f and type(f) == "function", err)
    return f(select(2, ...))
end

loadluafile("spickle2.lua")
loadluafile("data.lua", T)
loadluafile("scanning.lua", T)
loadluafile("events.lua", T)
loadluafile("ui.lua", T, loadluafile, check_number)
if (T.load_tab) then
	loadluafile("targetlist_tab.lua", T)
end


RegisterUserCommand("roiddb", function() ShowDialog(T.UI.maindlg) end)
RegisterUserCommand("roiddbscan", T.Scan.AutoScanStartStop)
RegisterUserCommand("roiddbdeletesectordata", function()
	T.DeleteSectorData()
	print(string.format("%sDeleting asteroid data for %s\127o", "\127FF0000", ShortLocationStr(GetCurrentSectorid())))
end)
