local T = ...
local ore_reference = T.ore_reference
-- TODO FIX header coloring when a non-primary header was previously selected and the user changes the view away and back
-- TODO FIX row highlting when sorting columns


local function get_control_width(str, font)
	local text = iup.label{title = str, font = font, expand = "NO"}
	iup.Map(text)
	local w, h = tonumber(text.w), tonumber(text.h)
	iup.Destroy(text)
	return w, h
end

local function check_filter(tbl, filter)
	if (filter == "All" or filter == nil) then
		return true
	elseif (tbl[filter]) then 
		return true 
	end
	return false
end

local matrix_toggles = {
-- 	[<id>] = <iup.toggle>,
}

local function is_pinned(id)
	local sectorid = T.LocalData.pinned and T.LocalData.pinned[T.UI.navmap:GetSectorID()]
	return sectorid and sectorid[id] and 1 or 0
end

local sort_dir = {
	--[1] = column index,
	--[2] = (true|false) -- false == ascending or same as defined;  true == descending or opposite of defined
}
local matrix_layout = {}
T.MatrixLayout = matrix_layout

-- ### ##################################################### ###
-- ### ##################################################### ###
-- ### ##################################################### ###
matrix_layout["sector_roid_list"] = {
	[1] = {
		title = "",
		alignment = "ACENTER",
		no_sort = true,
		sort_func = function(a, b) end,
		control = true,
	},
	[2] = {
		title = "ID",
		alignment = "ACENTER",
		width = function() return get_control_width("0000000", (matrix_layout["sector_roid_list"].font or Font.Default) * T.FONT_SCALE * .9) end,
		sort_dir = false, -- ascending
		sort_func = function(a, b)
			local A, B = is_pinned(a.id), is_pinned(b.id)
			if (A ~= B) then return A > B end

			A, B = a.id, b.id
			if (sort_dir[2]) then return A > B else return A < B end
		end
	},
	[2 + #ore_reference[1] + 1] = {
		title = "Distance",
		alignment = "ALEFT",
		width = function() return get_control_width("  Distance ", (matrix_layout["sector_roid_list"].font or Font.Default) * T.FONT_SCALE * .9) end,
		sort_dir = false, -- ascending
		sort_func = function(a, b)
			local A, B = is_pinned(a.id), is_pinned(b.id)
			if (A ~= B) then return A > B end

			A, B = a.id, b.id
			if (sort_dir[2]) then return A > B else return A < B end

		end,
	},
	highlight_sort_col = true,
	primary_column = 2,
	font = Font.Default,

	build_sorted = function(self, data, filter)
		local list = {}
		local count = 0
		local max = 0
		for roidID, roidtbl in pairs(data or {}) do
			if (check_filter(roidtbl.ore, filter)) then
				table.insert(list, roidtbl)
			end
			count = count + 1
			if (roidID > max) then max = roidID end
		end
		T.UI.scanned_total.title = count
		T.UI.est_max_total.title = max
		return list
	end,

	click = function(self, lin, col)
		if (T.UI.navmap:GetSectorID() == GetCurrentSectorid()) then
			T.UI.targetbtn.active = "YES"
		end
	end,

	double_click = function(self, lin, col, mode)
	-- Don't call with colon notation.  pass the matrix reference
		local nid, oid = radar.GetRadarSelectionID() -- save orginal target for retarget if necessary

		if (T.Scan.LockFunction(T.Scan.GetSectorNodeID)) then -- there may not be a detectable roid in range even after cycling.  don't attempt to target if there's no work to do
			gkinterface.GKProcessCommand("RadarNone")
			radar.SetRadarSelection(T.Scan.GetSectorNodeID(), self[lin..":2"])
			if (GetTargetInfo()) then
				HideDialog(iup.GetDialog(self))
			else
				T.MessageDlg:SetMessage("The selected asteroid is out of range.", "OK", function(self) HideDialog(iup.GetDialog(self)) end)
				ShowDialog(T.MessageDlg)
				radar.SetRadarSelection(nid, oid) -- retarget original target, if possible
			end
		else
			T.MessageDlg:SetMessage("No asteroids are within range.", "OK", function(self) HideDialog(iup.GetDialog(self)) end)
			ShowDialog(T.MessageDlg)
		end
	end,

	init = function(self, matrix)
		matrix["numcol"] = #self
		for i, t in ipairs(self) do
			matrix:setcell(0, i, t.title)
			matrix["width"..i] = type(t.width) == "function" and t.width() or t.width
			matrix["alignment"..i] = t.alignment
		end
		
		-- Reset the width of the pin column according to the width of a toggle
		local t = iup.stationtoggle{title = "", font = T.Font * .9}
		iup.Map(t)
		matrix["width1"] = t.w
		iup.Destroy(t)
	
	end,

	write_lin = function(self, matrix, lin, sort_col, data, cur_target)
		local sectortbl = T.LocalData.pinned[T.UI.navmap:GetSectorID()]
		matrix_toggles[data.id] = iup.stationtoggle{
			title = "", 
			value =  sectortbl and sectortbl[data.id] and "ON" or "OFF",
			font = matrix_layout["sector_roid_list"].font * T.FONT_SCALE * .9,
			action = function(self, v)
				if (v == 1) then 
					T.LocalData.pinned[T.UI.navmap:GetSectorID()] = T.LocalData.pinned[T.UI.navmap:GetSectorID()] or {}
					T.LocalData.pinned[T.UI.navmap:GetSectorID()][data.id] = true
				else
					T.LocalData.pinned[T.UI.navmap:GetSectorID()][data.id] = false
				end 
				SaveSystemNotes(spickle(T.LocalData), roiddb.notesid)
			end
		}
	
		matrix[lin..":1"] = matrix_toggles[data.id]
		matrix:setcell(lin, 2, data.id)
		for col, ore_type in ipairs(ore_reference[1]) do
			matrix:setcell(lin, col + 2, data["ore"][ore_type] or "")
		end
		-- Get the distance for the current roid
		if (T.UI.navmap:GetSectorID() == GetCurrentSectorid() and T.Scan.LockFunction(T.Scan.GetSectorNodeID)) then
			-- Use a timer to retarget the original target after the matrix has been populated.
			if (not T.Scan.timer:IsActive()) then
				local orig_nodeID, orig_objID = radar.GetRadarSelectionID()
				T.Scan.timer:SetTimeout(1, function() T.Scan.LockFunction(radar.SetRadarSelection, orig_nodeID, orig_objID) end)
			end

			-- Use LockFunction to lock the scan to prevent issues with target switching
			T.Scan.LockFunction(function()
				radar.SetRadarSelection() --  Clear the radar
				radar.SetRadarSelection(T.Scan.GetSectorNodeID(), data.id)
			end)
			local distance = math.floor(GetTargetDistance() or 9999)
			if (distance == 9999) then distance = "" end
			matrix:setcell(lin, 2 + #ore_reference[1] + 1, distance)
		else
			matrix:setcell(lin, 2 + #ore_reference[1] + 1, "-")
			--matrix["alignment"..lin..":"..(2 + #ore_reference[1] + 1)] = "ACENTER"
		end
		matrix:setcell(lin, #self, "") -- End of line filler.

		local i = lin % 2
		-- color the row in the alternating color scheme
		i = cur_target == tonumber(matrix[lin..":2"]) and 4 or i
		matrix:setattribute("BGCOLOR", lin, -1, T.BGColors[i])
		-- color the sort column
		matrix:setattribute("BGCOLOR", lin, sort_col, T.BGColors[i > 2 and i or 2])
		-- recolor the column containing the IUP control
		matrix:setattribute("BGCOLOR", lin, 1, "255 255 255 255")

	end,
}
-- columns for specific ore types.
for i, ore_type in ipairs(ore_reference[1]) do
	matrix_layout["sector_roid_list"][i+2] =
		{
			title = ore_type,
			alignment = "ACENTER",
			width = function() return get_control_width("0000.0", matrix_layout["sector_roid_list"].font * T.FONT_SCALE * .9) end,
			sort_dir = true, -- descending
			sort_func = function(a, b)
				local A, B = is_pinned(a.id), is_pinned(b.id)
				if (A ~= B) then return A > B end

				A, B = (a["ore"][ore_type] or 0), (b["ore"][ore_type] or 0)
				if (A == B) then A, B = b.id,  a.id end -- necessary to make IDs sort in the same direction as the ore percents
				if (sort_dir[2]) then return A > B else return A < B end
			end
		}
end
-- end of line filler
table.insert(matrix_layout["sector_roid_list"],
	{
		title = "",
		width = 0,
		no_sort = true,
		sort_func = function(a, b)
		end
	}
)

-- ### ##################################################### ###
-- ### ##################################################### ###
-- ### ##################################################### ###
matrix_layout["universe_ore_search"] = {
	[1] = {
		title = "Location",
		width = get_control_width(string.rep("X", 30)),
		alignment = "ALEFT",
		sort_func = function(a, b)
			local A, B = SystemNames[a[1]], SystemNames[b[1]]
			if (sort_dir[2]) then return A > B else return A < B end
		end
	},
	[2] = {
		title = "Highest Ore %",
		width = function() return get_control_width(string.rep("X", string.len("Highest Ore %")), (T.MatrixLayout["universe_ore_search"].font or Font.Default) * T.FONT_SCALE * .9) end,
		alignment = "ARIGHT",
		sort_dir = true, -- descending
		sort_func = function(a, b)
			local A, B
			A, B = a[2], b[2]
			if (A == B) then A, B = a[1], b[1] end
			if (sort_dir[2]) then return A > B else return A < B end
		end
	},
	highlight_sort_col = false,
	
	build_sorted = function(self, data, filter)
		local list = {}
		for systemid, systemtbl in pairs(data) do
			if (systemtbl[filter] and systemtbl[filter].highest >= (tonumber(T.UI.search_min.value) or 0)) then -- should I verify the presence of highest and count?
				table.insert(list, {systemid, systemtbl[filter].highest, systemtbl[filter].count})
			elseif (filter == "All") then -- All Ore Search
				local count = 0
				for ore_type, ore_data in pairs(systemtbl) do
					if (type(ore_type) ~= "number") then
						count = count + ore_data.count or 0
					end
				end
				table.insert(list, {systemid, "-", count})
			end
		end
	
		return list
	end,
	
	click = function(self, lin, col)
		if (IsConnected()) then
			--T.UI.search_setpathbtn.active = "YES"
		end
		T.UI.drilldownbtn.active = "YES"
	end,

	double_click = function(self, lin, col, mode)
		T.UI.drilldownbtn:action()
	end,


	init = function(self, matrix)
		matrix["numcol"] = #self
		matrix["numcol_visible"] = #self

		for i, t in ipairs(self) do
			matrix:setcell(0, i, t.title)
			matrix["width"..i] = type(t.width) == "function" and t.width() or t.width
			matrix["alignment"..i] = t.alignment or "ALEFT"
		end
	end,
                  
 	write_lin = function(self, matrix, lin, sort_col, data)
		matrix:setcell(lin, 1, string.format("%s (%d roids)", SystemNames[data[1]], data[3]))
		matrix:setcell(lin, 2, data[2])

		local i = lin % 2
		matrix:setattribute("BGCOLOR", lin, -1, T.BGColors[i])
	end,
		
	
}

-- ### ##################################################### ###
-- ### ##################################################### ###
-- ### ##################################################### ###
matrix_layout["system_ore_search"] = {
	[1] = {
		title = "Location",
		width = function() return get_control_width(string.rep("X", 30), (matrix_layout["system_ore_search"].font or Font.Default) * T.FONT_SCALE * .9) end,
		alignment = "ALEFT",
		sort_func = function(a, b)
			local A, B = ShortLocationStr(a[1]), ShortLocationStr(b[1])
			if (sort_dir[2]) then return A > B else return A < B end
		end
	},
	[2] = {
		title = "Highest Ore %",
		width = function() return get_control_width(string.rep("X", string.len("Highest Ore %")), (matrix_layout["system_ore_search"].font or Font.Default) * T.FONT_SCALE * .9) end,
		alignment = "ARIGHT",
		sort_dir = true, -- descending
		sort_func = function(a, b)
			local A, B
			local filter = a[2]
			A, B = a[2], b[2]
			if (A == B) then A, B = a[1], b[1] end
			if (sort_dir[2]) then return A > B else return A < B end
		end
	},
	highlight_sort_col = false,
	
	build_sorted = function(self, data, filter)
		local list = {}
		for sectorid, sectortbl in pairs(data) do
			if (sectortbl[filter] and sectortbl[filter].highest >= (tonumber(T.UI.search_min.value) or 0)) then -- should I verify the presence of highest and count?
				table.insert(list, {sectorid, sectortbl[filter].highest, sectortbl[filter].count})
			elseif (filter == "All" and tonumber(sectorid)) then -- All Ore Search
				local count = 0
				for ore_type, ore_data in pairs(sectortbl) do
					count = count + ore_data.count or 0
				end
				table.insert(list, {sectorid, "-", count})
			end
		end

		return list
	end,
	
	click = function(self, lin, col)
		if (IsConnected()) then
			T.UI.search_setpathbtn.active = "YES"
		end
		T.UI.drilldownbtn.active = "YES"
		T.UI.search_backbtn.active = "YES"
	end,

	double_click = function(self, lin, col, mode)
		T.UI.drilldownbtn:action()
	end,


	init = function(self, matrix)
		matrix["numcol"] = #self
		matrix["numcol_visible"] = #self

		for i, t in ipairs(self) do
			matrix:setcell(0, i, t.title)
			matrix["width"..i] = type(t.width) == "function" and t.width() or t.width
		end
	end,
                  
 	write_lin = function(self, matrix, lin, sort_col, data)
		matrix:setcell(lin, 1, string.format("%s (%d roids)", ShortLocationStr(data[1]), data[3]))
		matrix:setcell(lin, 2, data[2])

		local i = lin % 2
		matrix:setattribute("BGCOLOR", lin, -1, T.BGColors[i])

	end,
		
	
}

-- Two ways (?) to set the value of a cell
-- matrix["<L>:<C>"] = <value>
-- matrix:setcell(<L>, <C>, <value>)
-- matrix:setattribute(<attribute>, <L>, <C>, <value>) -- <attribute> == "" sets the value of the cell / width can't be set

-- matrix:fgcolor_cb(line, col) -> R, G, B, A, ret: number) -- ret (iup.DEFAULT, iup.IGNORE)
-- matrix:bgcolor_cb(line, col) -> R, G, B, A, ret: number) -- ret (iup.DEFAULT, iup.IGNORE)


-- ### ##################################################### ###
-- ### ##################################################### ###
-- ### ##################################################### ###
function T:BuildMatrix(layout, properties)
	local cur_sort_col
	local cur_data
	local cur_filter
	local sel_id
	local sel_lin
	local cur_lin
	local cur_target

	properties = properties or {}
	properties.font = (layout.font or Font.Default) * T.FONT_SCALE * .9-- not sure i like this approach
	--local matrix_desc = iup.label{title = "", expand = "HORIZONTAL", font = Font.H1 * T.FONT_SCALE}
	local matrix = iup.matrix(properties)
	local no_roids = iup.label{title = "There is no asteroid information to display.", visible = "NO", font = T.Font}
	local matrix_container = iup.vbox{
-- 		iup.stationhighopacitysubframe{
-- 			matrix_desc,
-- 		},
		iup.pdarootframe{
			iup.zbox{
				iup.vbox{
					matrix,
				},
				no_roids,
				alignment = "ACENTER",
				all = "YES",
			},
		},
		gap = 10,
	}

	function matrix:SetDescriptionMsg(s)
		T.UI.header_desc.title = s or ""
	end
	
	function matrix:GetCellInCurLine(col)
		return matrix:getcell(cur_lin, col)
	end

	-- Populates the matrix after a slight delay during refresh.  
	local function populate_matrix(self, col, filter)
		local sorted_list = layout:build_sorted(cur_data, filter)
		table.sort(sorted_list, layout[col].sort_func)
		no_roids.visible = #sorted_list == 0 and "YES" or "NO"

		-- fill the matrix
		self["numlin"] = #sorted_list
		for lin, lin_data in ipairs(sorted_list) do
			layout:write_lin(self, lin, col, lin_data, cur_target)
		end

		-- if a line was previously selected, try to reselect it
		if (sel_id) then
			for lin, roidtbl in ipairs(sorted_list) do
				if (roidtbl.id == sel_id) then
					sel_lin = lin
					sel_id = nil

					-- self:click_cb(sel_lin, 1) -- why?
					-- highlight the originally selected line
					sel_lin = sel_lin, self:enteritem_cb(sel_lin) -- preserve the value of sel_lin, which is cleared in enteritem
					cur_lin = sel_lin

					break
				end
			end
		end
		cur_filter = filter or cur_filter
	end		
	
	function matrix:refresh(col, filter)
		--if (not cur_data) then return false end -- nothing to do.  get out of here.
		col = col or cur_sort_col or 1
		filter = filter or cur_filter
		-- Get the id of the currently selected line
		if (cur_lin) then sel_id = tonumber(self:GetCellInCurLine(layout.primary_column or 1)) end

		local togs = matrix_toggles
		matrix_toggles = {}

		-- Delete all lines
		self["dellin"] = "1--1"

		-- destroy all pinned roid toggles
		for id, c in pairs(matrix_toggles) do
			iup.Destroy(c)
		end
		
		-- Give the matrix time to properly reset after a clear.  Forces the focus back to the first line.
		Timer():SetTimeout(1, function() populate_matrix(self, col, filter) end)
	end	
	
	local function set_sort_col(lin, col)
		matrix:setattribute("FGCOLOR", lin, cur_sort_col or 0, "255 255 255")
		matrix:setattribute("FGCOLOR", lin, col, "255 0 0")
	
		if (sort_dir[1] ~= col) then 
			sort_dir = {col, layout[col].sort_dir}
		else
			sort_dir[2] = not sort_dir[2]
		end
		cur_sort_col = col
	end
	
	-- Resets the matrix data, filter, and layout if specified
	function matrix:reset(new_data, new_filter, new_layout)
		local sort_col = layout.primary_column or 1
		local same_data = cur_data and cur_data == new_data
		cur_target = T.UI.navmap:GetSectorID() == GetCurrentSectorid() and ({radar.GetRadarSelectionID()})[2] or nil

		if (false) then --same_data) then  -- Is avoiding work necessary in this application?  doing this makes it harder to handle the targetbtn active state
		-- TODO Why did I make this an auto-fail condition and not just remove it?  It seems to work.
			if (cur_filter and cur_filter == new_filter) then 
				return -- No reason to change anything if the same sector and filter is being viewed.
			else
				sort_col = cur_sort_col
			end
		else
			cur_lin = nil
			cur_data = new_data or {}
			layout = new_layout or layout -- TODO Can the columns be changed after mapping?
		end 
		cur_filter = new_filter or "All"

		no_roids.visible = new_data and "NO" or "YES"
		--if (not new_data) then return end -- if no data, return
		
		sort_dir = {} -- reset the sortin'
		set_sort_col(0, sort_col)
		self:refresh(sort_col, cur_filter)
	end
    

	function matrix:click_cb(lin, col, mouse_status)
		if (lin == 0 and not layout[col].no_sort) then
			set_sort_col(lin, col)
			--if (cur_lin) then sel_id = self:GetCellInCurLine(layout.primary_column or 1) end
			
			self:refresh(col)
			--cur_lin = nil
		else
			cur_lin = lin
			if (layout.click) then layout:click(lin, col, cur_filter) end
		end
	end
	
-- 	function matrix:fgcolor_cb(lin, col)
-- 	end
-- 	matrix.bgcolor_cb = matrix.fgcolor_cb
	function matrix:bgcolor_cb(lin, col)
		-- Use to paint the entire matrix
		-- To repaint a line for selection must be done with enter/leaveitem_cb()
		local i = lin % 2
		if     (lin == cur_lin)      then i = 2
		elseif ((tonumber(self[lin..":2"]) or "-") == cur_target) then i = 4
		elseif (layout.highlight_sort_col and col == cur_sort_col) then i = 2
		elseif (layout[col].control) then i = 3
		end
		return T.BGColors.Numbers[i][1], T.BGColors.Numbers[i][2], T.BGColors.Numbers[i][3], T.BGColors.Numbers[i][4], iup.DEFAULT
	end
	
	function matrix:enteritem_cb(lin, col)
		if (sel_lin) then
			--self:leaveitem_cb(sel_lin)
			sel_lin = nil
		end
		self:setattribute("BGCOLOR", lin, -1, T.BGColors[2])
		for _, t in ipairs(layout) do
			if (t.control) then 
				self:setattribute("BGCOLOR", lin, 1, "255 255 255 255")
			end
		end
		return iup.DEFAULT
	end
	
	function matrix:leaveitem_cb(lin, col)
		lin = cur_lin or lin
		local i = lin % 2
		i = cur_target == (tonumber(self[lin..":2"]) or "-") and 4 or i
		self:setattribute("BGCOLOR", lin, -1, T.BGColors[i])

		if (layout.highlight_sort_col) then
			i = cur_target == (tonumber(self[lin..":2"]) or "-") and 4 or 2
			self:setattribute("BGCOLOR", lin, cur_sort_col, T.BGColors[i])
		end

		for _, t in ipairs(layout) do
			if (t.control) then
				self:setattribute("BGCOLOR", lin, 1, "255 255 255 255")
			end
		end

		cur_lin = nil
		return iup.DEFAULT
	end

	function matrix:edition_cb(lin, col, mode)
		if (layout.double_click) then layout.double_click(self, lin or cur_lin, col, mode) end
		return iup.IGNORE
	end
	
	layout:init(matrix)

	return matrix_container, matrix
end



--[=[

When Refresh is Called

reset
 - on page change (sector change)
     * if data same, keep sort col

 
filter change
 - at sector view

sort column change


]=]


--[=[
event progression

Clicking line; no line currently selected.
leaveitem -> enteritem -> click

Clicking line; line already selected.
leaveitem -> enteritem -> click

Clicking header; line selected
click -> refresh -> enteritem

clicking header; previous line highlighted; no new line clicked
click -> refresh

clicking header; previous line highlighted; new line clicked
click -> refresh -> enteritem




]=]


